#!/usr/bin/python

from commandgenerator import CommandGenerator

class CustomController:
    """
    Controller is defined by its IP address and port. It is provided to
    configured switches.
    """
    def __init__(self, name, ip, port):
        self.name = name
        self.ip = ip
        self.port = port

class Switch:
    def __init__(self, name, bw, controller, isEdge = False):
        self.name = name
        self.bw = bw
        self.controller = controller
        self.edge = isEdge

class AutonomousSystem:
    """
    Autonomous system (AS) is a set of switches with links between these switches.
    Each AS has a name.
    """
    def __init__(self, name, switches, links):
        self.name = name
        self.switches = switches
        self.links = links

class CommunicationFlow:
    """
    Parent class for each data flow. Since it is expected for all flows to have
    server and client side and have a method for each. In case new flows are
    introduced, these must be overriden to provide command string that will be
    executed on both sides.
    """
    def __init__(self):
       self.FlowType = []

    def generate_server_command(self, simulators_path, output_path):
        return ""

    def generate_client_command(self, simulators_path, output_path, ip):
        return "" 

class StreamFlow( CommunicationFlow ):
    """
    Stream flow represents data flow from server to client for constant data
    flow.
    """

    CONSTANT_LOAD = 'constant.load'
    VARIABLE_LOAD = 'variable.load'
    LOADPROFILE_LOAD = 'loadprofile.load'
    DRONE1_LOAD = 'drone1.load'
    DRONE2_LOAD = 'drone2.load'

    def __init__(self, max_bandwidth, source, destination, load_type):
        self.max_bandwidth = max_bandwidth
        "Tuple<switch, port>"
        self.source = source
        "Tuple<switch, >"
        self.destination = destination
        "Load type represents load file name expected at default folder."
        self.load_type = load_type

    def generate_server_command(self, simulators_path, output_path):
        port = self.source[1]
        command = CommandGenerator.stream_traffic_generator(simulators_path, output_path, port, self.max_bandwidth, self.source[0].name, self.load_type)
        return command

    def generate_client_command(self, simulators_path, output_path, ip):
        port = self.source[1]
        command = CommandGenerator.stream_traffic_sink(simulators_path, output_path, ip, port, self.source[0].name)
        return command

class CommandFlow( CommunicationFlow ):
    """
    CommandFlow represents flow that emulates request-response communication
    (sending a command and receiving response). It can be configured to specify
    command and response sizes, command number and period of commands. In case
    period is not defines, new command will be issued as soon as previous response
    is received.
    """
    def __init__(self, execution_time, commanding_period, command_size, commands_number, response_size, source, destination):
        self.execution_time = execution_time
        self.commanding_period = commanding_period
        self.command_size = command_size
        self.commands_number = commands_number
        self.response_size = response_size
        self.source = source
        self.destination = destination
        
    def generate_server_command(self, simulators_path, output_path):
        port = self.source[1]
        command = CommandGenerator.device_simulator(simulators_path, port, self.source[0].name, output_path)
        return command
        
    def generate_client_command(self, simulators_path, output_path, ip):
        port = self.source[1]
        command = CommandGenerator.command_simulator(simulators_path,
                                                     output_path,
                                                     self.execution_time,
                                                     self.commanding_period,
                                                     self.command_size,
                                                     self.commands_number,
                                                     ip,
                                                     port,
                                                     self.destination[0].name)
        return command

class EvaluationParameters:
    """
    Test parameters are separated in simulation parameters class to ease execution
    of multiple tests on the same topology and flows.
    """
    def __init__(self, duration, test_name, use_netflow, nodes_path):
        self.duration = duration;
        self.test_name = test_name
        self.use_netflow = use_netflow
        self.nodes_path = nodes_path

class SGCommunicationTopology:
    """
    SG communication topology includes network topology with SG flows defined.
    """
    AutonomousSystems = None
    links = None
    flows = None

    def Switches(self):
        switches = []
        for autonomous_system in self.AutonomousSystems:
            switches.extend(autonomous_system.switches)
        return switches
 
class SGCommunicationModel:
    """
    SG Communication Model includes SG communication topology and simulation parameters.
    """
    def __init__(self, topology, parameters):
        self.topology = topology
        self.parameters = parameters
